import pygame
import random
import math

class Particle:
    def __init__(self, x, y, radius, color):
        self.x = x
        self.y = y
        self.radius = radius
        self.color = color
        self.speed = random.uniform(1, 5)
        self.angle = random.uniform(0, 2 * math.pi)
        self.path = []

    def move(self):
        self.x += self.speed * math.cos(self.angle)
        self.y += self.speed * math.sin(self.angle)
        self.path.append((int(self.x), int(self.y)))  # Record the position in the path


    def check_collision(self, other_particle):
        distance = math.sqrt((self.x - other_particle.x)**2 + (self.y - other_particle.y)**2)
        return distance <= self.radius + other_particle.radius

    def handle_collision(self, other_particle):
        angle_diff = math.atan2(other_particle.y - self.y, other_particle.x - self.x)
        total_mass = self.speed + other_particle.speed
        self.angle = angle_diff + math.pi
        other_particle.angle = angle_diff
        self.speed = total_mass * (other_particle.speed / total_mass)
        other_particle.speed = total_mass * (self.speed / total_mass)

    def check_wall_collision(self, width, height):
        if self.x - self.radius < 0 or self.x + self.radius > width:
            self.angle = math.pi - self.angle
        if self.y - self.radius < 0 or self.y + self.radius > height:
            self.angle = -self.angle

def create_particles(num_particles):
    particles = []
    for _ in range(num_particles):
        x = random.uniform(radius, width - radius)
        y = random.uniform(radius, height - radius)
        color = (255,0,0)
        particles.append(Particle(x, y, radius, color))
    return particles
  
def create_particle_big(num_particles):
    particles = []
    for _ in range(num_particles):
        x = width/2 #random.uniform(radius_big, width - radius_big)
        y = height/2 #random.uniform(radius_big, height - radius_big)
        color = (0,0,255)
        particles.append(Particle(x, y, radius_big, color))
    return particles
  
pygame.init()

width, height = 600, 600
radius = 2
radius_big = 25
num_particles = 350
fps = 20

particles = create_particles(num_particles)
particle_big = create_particle_big(1)

selected_particle = particle_big[0]

screen = pygame.display.set_mode((width, height))
pygame.display.set_caption("Particle Simulation")

clock = pygame.time.Clock()
running = True
m = 0
while running:
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            running = False

    for particle in particles:
        particle.move()
        particle.check_wall_collision(width, height)
        
    for particle in particle_big:
        particle.move()
        particle.check_wall_collision(width, height)

    for i in range(len(particles)):
        for j in range(i+1, len(particles)):
            if particles[i].check_collision(particles[j]):
                particles[i].handle_collision(particles[j])
        if particles[i].check_collision(particle_big[0]):
                particles[i].handle_collision(particle_big[0])
        if particle_big[0].check_collision(particles[i]):
            particle_big[0].handle_collision(particles[i])
    

    screen.fill((255, 255, 255))
    for particle in particles:
        pygame.draw.circle(screen, particle.color, (int(particle.x), int(particle.y)), particle.radius)
    for particle in particle_big:
        pygame.draw.circle(screen, particle.color, (int(particle.x), int(particle.y)), particle.radius)

    if m>4:
      pygame.draw.lines(screen, (0,0,255), False, selected_particle.path, 2)
    
    
    pygame.display.flip()
    clock.tick(fps)
    m = m+1
pygame.quit()